// ----------------------------------------------------------------------------
//
//  Copyright (C) 2008-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <string.h>
#include <math.h>
#include "gainctl.h"


Gainctl::Gainctl (void) :
    _touch0 (0),
    _touch1 (0),
    _state (BYPASS),
    _g0 (1),
    _g1 (1),
    _g  (1),
    _dg (0)
{
}


Gainctl::~Gainctl (void)
{
}


void Gainctl::prepare (int nsamp)
{
    if (_touch1 != _touch0)
    {
        if (_g1 != _g0)
	{
 	    if      (_g0 > 2 * _g1) _g1 *= 2;
	    else if (_g1 > 2 * _g0) _g1 /= 2;
	    else                    _g1 = _g0;
	    _dg = (_g1 - _g) / nsamp;
	    if (fabsf (_dg) < 1e-30f)
	    {
		_g = _g1;
                _dg = 0;
		_state = STATIC;
	    }
	    else _state = SMOOTH;
	}
	else
	{
	    _touch1 = _touch0;
            _state = (fabs (_g - 1) < 0.001f) ? BYPASS : STATIC;
	}
    }
}


void Gainctl::process (int nsamp, int nchan, float *inp[], float *out[])
{
    int     i, j;
    float   g;
    float   *p, *q;

    g = _g;
    for (i = 0; i < nchan; i++)
    {
        p = inp [i];
        q = out [i];
	if (_state == SMOOTH)
	{
	    g = _g;
	    for (j = 0; j < nsamp; j++)
	    {
		g += _dg;
		*q++ = g * *p++;
	    }
	}
	else if (_state == STATIC)
	{
	    g = _g;
	    for (j = 0; j < nsamp; j++)
	    {
		*q++ = g * *p++;
	    }
	}
	else
	{
	    if (p != q) memcpy (q, p, nsamp * sizeof (float));
	}
    }
    if (_state == SMOOTH) _g = g;
}
