// ----------------------------------------------------------------------------
//
//  Copyright (C) 2008-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "styles.h"
#include "global.h"
#include "mainwin.h"


Mainwin::Mainwin (X_rootwin *parent, X_resman *xres, int xp, int yp, Jclient *jclient) :
    A_thread ("Main"),
    X_window (parent, xp, yp, XSIZE, YSIZE, XftColors [C_MAIN_BG]->pixel),
    _stop (false),
    _xres (xres),
    _jclient (jclient)
{
    X_hints     H;
    int         i, x, y;

    _atom = XInternAtom (dpy (), "WM_DELETE_WINDOW", True);
    XSetWMProtocols (dpy (), win (), &_atom, 1);
    _atom = XInternAtom (dpy (), "WM_PROTOCOLS", True);

    H.position (xp, yp);
    H.minsize (XSIZE, YSIZE);
    H.maxsize (XSIZE, YSIZE);
    H.rname (xres->rname ());
    H.rclas (xres->rclas ());
    x_apply (&H); 
    x_set_title (xres->rname ());

    RotaryCtl::init (disp ());
    x = 0;
    y = 0;
    _rotary [HPFIL] = new Rlogctl (this, this, HPFIL, &rg_hpfil, x, y, 120, 8,  10.0, 320.0, 80.0);
    _rotary [EGAIN] = new Rlinctl (this, this, EGAIN, &rg_mgain, x, y, 150, 5, -18.0,  18.0,  0.0);
    _butt_hpf = new Pbutt2 (this, this, B_HPF, bhpfil, x + 10, y + 9, 24, 24);
    _butt_hpf->x_map ();
    _butt_equ = new Pbutt2 (this, this, B_EQU, bequal, x + 10, y + 60, 24, 24);
    _butt_equ->x_map ();
    x += 92;
    _rotary [GAIN1] = new Rgain18 (this, this, GAIN1, &rg_sgain, x, y);
    _rotary [FREQ1] = new Rlogctl (this, this, FREQ1, &rg_sfreq, x, y,  96, 4, 2.5e1, 4e2, 5e1);
    _rotary [TYPE1] = new Rlinctl (this, this, TYPE1, &rg_sbndw, x, y, 100, 5, -1.0, 1.0, 0.0);
    x += 100;
    _rotary [GAIN2] = new Rgain18 (this, this, GAIN2, &rg_sgain, x, y);
    _rotary [FREQ2] = new Rlogctl (this, this, FREQ2, &rg_sfreq, x, y, 96, 4, 20.0,  5e3, 315);
    _rotary [BNDW2] = new Rlogctl (this, this, BNDW2, &rg_sbndw, x, y, 90, 5, 1.0/16, 4.0, 1.0);
    x += 100;
    _rotary [GAIN3] = new Rgain18 (this, this, GAIN3, &rg_sgain, x, y);
    _rotary [FREQ3] = new Rlogctl (this, this, FREQ3, &rg_sfreq, x, y, 96, 4, 40.0, 10e3, 630);
    _rotary [BNDW3] = new Rlogctl (this, this, BNDW3, &rg_sbndw, x, y, 90, 5, 1.0/16, 4.0, 1.0);
    x += 100;
    _rotary [GAIN4] = new Rgain18 (this, this, GAIN4, &rg_sgain, x, y);
    _rotary [FREQ4] = new Rlogctl (this, this, FREQ4, &rg_sfreq, x, y, 96, 4, 80.0, 20e3, 1250);
    _rotary [BNDW4] = new Rlogctl (this, this, BNDW4, &rg_sbndw, x, y, 90, 5, 1.0/16, 4.0, 1.0);
    x += 100;
    _rotary [GAIN5] = new Rgain18 (this, this, GAIN5, &rg_sgain, x, y);
    _rotary [FREQ5] = new Rlogctl (this, this, FREQ5, &rg_sfreq, x, y,  96, 4 , 1e3, 16e3, 8e3);
    _rotary [TYPE5] = new Rlinctl (this, this, TYPE5, &rg_sbndw, x, y, 100, 5, -1.0, 1.0f, 0.0);
    x += 100;
    for (i = 0; i < NROTARY; i++) _rotary [i]->x_map ();

    x = 92;
    y = 0;
    for (i = 0; i < 5; i++)
    {
        _tgain [i] = new X_textip (this, 0, &tsgain, x + 41, y + 2, 44, 12, 15);
	_tgain [i]->set_align (1);
	_tgain [i]->set_text (" 0.0");
	_tgain [i]->x_map ();
        _tfreq [i] = new X_textip (this, 0, &tsfreq, x + 1, y + 2, 44, 12, 15);
	_tfreq [i]->set_align (0);
	_tfreq [i]->x_map ();
	x += 100;
    }

    _fsamp = jclient->fsamp ();
    setparam (HPFIL);
    setparam (EGAIN);
    setparam (FREQ1);
    setparam (FREQ2);
    setparam (FREQ3);
    setparam (FREQ4);
    setparam (FREQ5);
    x_add_events (ExposureMask); 
    x_map (); 
    set_time (0);
    inc_time (250000);
}

 
Mainwin::~Mainwin (void)
{
    RotaryCtl::fini ();
}

 
int Mainwin::process (void)
{
    int e;

    if (_stop) handle_stop ();

    e = get_event_timed ();
    switch (e)
    {
    case EV_TIME:
        handle_time ();
	break;
    }
    return e;
}


void Mainwin::handle_event (XEvent *E)
{
    switch (E->type)
    {
    case Expose:
	expose ((XExposeEvent *) E);
	break;  
 
    case ClientMessage:
        clmesg ((XClientMessageEvent *) E);
        break;
    }
}


void Mainwin::expose (XExposeEvent *E)
{
    if (E->count) return;
    redraw ();
}


void Mainwin::clmesg (XClientMessageEvent *E)
{
    if (E->message_type == _atom) _stop = true;
}


void Mainwin::handle_time (void)
{
    inc_time (250000);
}


void Mainwin::handle_stop (void)
{
    put_event (EV_EXIT, 1);
}


void Mainwin::handle_callb (int type, X_window *W, XEvent *E)
{
    PushButton *B;
    RotaryCtl  *R;
    int         k;
    
    switch (type)
    {
    case RotaryCtl::DELTA:
	R = (RotaryCtl *) W;
	k = R->cbind ();
        setparam (k);
	break;

    case PushButton::PRESS:
	B = (PushButton *) W;
	k = B->cbind ();
	switch (k)
	{
	case B_HPF:
	    setparam (HPFIL);
	    break;
	case B_EQU:
	    setparam (EGAIN);
	    break;
	break;
	}
    }
}


void Mainwin::redraw (void)
{
    X_draw D (dpy (), win (), dgc (), 0);
    int x, y;

    D.setcolor (XftColors [C_MAIN_DL]);

    x = y = 0;
    XPutImage (dpy (), win (), dgc (), mastersect, 0, 0, x, y, 100, 100);
    D.move (x, 49);
    D.draw (x + 91, 49);
    x += 92;
    XPutImage (dpy (), win (), dgc (), lfshfsect, 0, 0, x, y, 100, 100);
    D.move (x, 1);
    D.draw (x, 99);
    x += 100;
    XPutImage (dpy (), win (), dgc (), parm1sect, 0, 0, x, y, 100, 100);
    D.move (x, 1);
    D.draw (x, 99);
    x += 100;
    XPutImage (dpy (), win (), dgc (), parm2sect, 0, 0, x, y, 100, 100);
    D.move (x, 1);
    D.draw (x, 99);
    x += 100;
    XPutImage (dpy (), win (), dgc (), parm3sect, 0, 0, x, y, 100, 100);
    D.move (x, 1);
    D.draw (x, 99);
    x += 100;
    XPutImage (dpy (), win (), dgc (), hfshfsect, 0, 0, x, y, 100, 100);
    D.move (x, 1);
    D.draw (x, 99);
    x += 100;
    XPutImage (dpy (), win (), dgc (), redzita, 0, 0, x, y, 47, 100);
    D.move (x, 1);
    D.draw (x, 99);
    D.move (x, 68);
    D.draw (x + 47, 68);
}


void Mainwin::fmtfreq (double v, char *s)
{
    const char *f;

    if      (v <= 3e1) f = "%1.1lf";
    else if (v <= 1e3) f = "%1.0lf";
    else if (v <= 3e3)
    {
	f = "%1.2lfk";
	v *= 1e-3;
    }
    else
    {
	f = "%1.1lfk";
	v *= 1e-3;
    }
    sprintf (s, f, v);
}


float Mainwin::db2lin (float g)
{
    return powf (10.0f, 0.05f * g);
}


void Mainwin::setparam (int k)
{
    int   i, c;
    float g, f, v;
    char  s [16];

    switch (k)
    {
    case HPFIL:
        c = _butt_hpf->state () & 2;
        f = _rotary [HPFIL]->value () / _fsamp;
        _jclient->set_hpfilt (c, f);
        break;
    case EGAIN:
        c = _butt_equ->state () & 2;
        g = db2lin (_rotary [EGAIN]->value ()); 
        _jclient->set_equal (c, g);
        break;
    case GAIN1:
    case FREQ1:
    case TYPE1:
	g = db2lin (_rotary [GAIN1]->value ());
	f = _rotary [FREQ1]->value () / _fsamp;
	v = _rotary [TYPE1]->value ();
        _jclient->set_param (0, g, f, v);
        break;
    case GAIN2:
    case FREQ2:
    case BNDW2:
	g = db2lin (_rotary [GAIN2]->value ());
	f = _rotary [FREQ2]->value () / _fsamp;
	v = _rotary [BNDW2]->value ();
	_jclient->set_param (1, g, f, v);
	break;
    case GAIN3:
    case FREQ3:
    case BNDW3:
	g = db2lin (_rotary [GAIN3]->value ());
	f = _rotary [FREQ3]->value () / _fsamp;
	v = _rotary [BNDW3]->value ();
	_jclient->set_param (2, g, f, v);
	break;
    case GAIN4:
    case FREQ4:
    case BNDW4:
	g = db2lin (_rotary [GAIN4]->value ());
	f = _rotary [FREQ4]->value () / _fsamp;
	v = _rotary [BNDW4]->value ();
	_jclient->set_param (3, g, f, v);
	break;
    case GAIN5:
    case FREQ5:
    case TYPE5:
	g = db2lin (_rotary [GAIN5]->value ());
	f = _rotary [FREQ5]->value () / _fsamp;
	v = _rotary [TYPE5]->value ();
        _jclient->set_param (4, g, f, v);
    }

    switch (k)
    {
    case FREQ1:	i = 0; break;
    case FREQ2:	i = 1; break;
    case FREQ3:	i = 2; break;
    case FREQ4:	i = 3; break;
    case FREQ5:	i = 4; break;
    default: i = -1;
    }
    if (i >= 0)
    {
	fmtfreq (_rotary [k]->value (), s);
	_tfreq [i]->set_text (s);
    }

    switch (k)
    {
    case GAIN1:	i = 0; break;
    case GAIN2:	i = 1; break;
    case GAIN3:	i = 2; break;
    case GAIN4:	i = 3; break;
    case GAIN5:	i = 4; break;
    default: i = -1;
    }
    if (i >= 0)
    {
	v = _rotary [k]->value ();
	if (fabs (v) < 0.1) _tgain [i]->set_text ("  0.0");
	else
	{
	    sprintf (s, "%+3.1lf", _rotary [k]->value ());
  	    _tgain [i]->set_text (s);
	}
    }
}



